# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tts.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ttts.proto\x12\x0c\x63hat_bot.tts\"K\n\x10LoadModelRequest\x12\x0f\n\x07tts_tag\x18\x01 \x01(\t\x12\x11\n\tis_reload\x18\x02 \x01(\x08\x12\x13\n\x0bjson_kwargs\x18\x03 \x01(\t\"\x13\n\x11LoadModelResponse\"%\n\x11SynthesizeRequest\x12\x10\n\x08tts_text\x18\x01 \x01(\t\"\'\n\x12SynthesizeResponse\x12\x11\n\ttts_audio\x18\x01 \x01(\x0c\x32\xac\x01\n\x03TTS\x12N\n\tLoadModel\x12\x1e.chat_bot.tts.LoadModelRequest\x1a\x1f.chat_bot.tts.LoadModelResponse\"\x00\x12U\n\x0cSynthesizeUS\x12\x1f.chat_bot.tts.SynthesizeRequest\x1a .chat_bot.tts.SynthesizeResponse\"\x00\x30\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tts_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_LOADMODELREQUEST']._serialized_start=27
  _globals['_LOADMODELREQUEST']._serialized_end=102
  _globals['_LOADMODELRESPONSE']._serialized_start=104
  _globals['_LOADMODELRESPONSE']._serialized_end=123
  _globals['_SYNTHESIZEREQUEST']._serialized_start=125
  _globals['_SYNTHESIZEREQUEST']._serialized_end=162
  _globals['_SYNTHESIZERESPONSE']._serialized_start=164
  _globals['_SYNTHESIZERESPONSE']._serialized_end=203
  _globals['_TTS']._serialized_start=206
  _globals['_TTS']._serialized_end=378
# @@protoc_insertion_point(module_scope)
